
use Test::More qw/no_plan/;
use Test::Differences;
use Carp::Assert;
use lib 'lib';
use strict;

BEGIN { 
  use_ok('CGI::Uploader');
  use_ok('Digest::MD5');
  use_ok('File::Path');
  use_ok('DBI');
  use_ok('CGI');
};

%ENV = (
  %ENV,
  'SCRIPT_NAME' => '/test.cgi',
  'SERVER_NAME' => 'perl.org',
  'HTTP_CONNECTION' => 'TE, close',
  'REQUEST_METHOD' => 'POST',
  'SCRIPT_URI' => 'http://www.perl.org/test.cgi',
  'CONTENT_LENGTH' => '2986',
  'SCRIPT_FILENAME' => '/home/usr/test.cgi',
  'SERVER_SOFTWARE' => 'Apache/1.3.27 (Unix) ',
  'HTTP_TE' => 'deflate,gzip;q=0.3',
  'QUERY_STRING' => '',
  'REMOTE_PORT' => '1855',
  'SERVER_PORT' => '80',
  'REMOTE_ADDR' => '127.0.0.1',
  'CONTENT_TYPE' => 'multipart/form-data; boundary=xYzZY',
  'SERVER_PROTOCOL' => 'HTTP/1.1',
  'PATH' => '/usr/local/bin:/usr/bin:/bin',
  'REQUEST_URI' => '/test.cgi',
  'GATEWAY_INTERFACE' => 'CGI/1.1',
  'SCRIPT_URL' => '/test.cgi',
  'SERVER_ADDR' => '127.0.0.1',
  'DOCUMENT_ROOT' => '/home/develop',
  'HTTP_HOST' => 'www.perl.org'
);

use CGI;
open(IN,'<t/upload_post_text.txt') || die 'missing test file';
binmode(IN);

*STDIN = *IN;
my $q = new CGI;

use vars qw($dsn $user $password);
my $file ='t/cgi-uploader.config';
my $return;
unless ($return = do $file) {
  warn "couldn't parse $file: $@" if $@;
  warn "couldn't do $file: $!"    unless defined $return;
  warn "couldn't run $file"       unless $return;
}
ok($return, 'loading configuration');


my $DBH =  DBI->connect($dsn,$user,$password);
ok($DBH,'connecting to database'), 

# create uploads table
my $drv = $DBH->{Driver}->{Name};

ok(open(IN, "<create_uploader_table.".$drv.".sql"), 'opening SQL create file');
my $sql = join "\n", (<IN>);
my $created_up_table = $DBH->do($sql);
ok($created_up_table, 'creating uploads table');


# Setup a new uploader object
my %imgs = (
  '300x300_gif' => {},
  '100x100_gif' => {},
);

my $u = 	CGI::Uploader->new(
  updir_path=>'t/uploads',
  updir_url=>'http://localhost/test',
  name_scheme => 'file_name',
  dbh  => $DBH,
  spec => \%imgs,
  query => $q,
);
ok($u, 'Uploader object creation');

# Now, let's do some testing
#use Data::Dumper;
my $form_data = $q->Vars;
#warn Dumper($u);

# Store a file
my ($entity);
eval { $entity = $u->store_uploads($form_data) };
is($@,'', 'calling store_uploads');
#warn Dumper("Entity => ", $entity);

# Get the id of the upload field
my $id = $entity->{'300x300_gif_id'};
ok($id, 'Record id is retrieved for 300x300_gif');

# check the name of the stored file via get_upload_record
my $rec = $u->get_upload_record($id);
my $filename = $rec->{'file_name'};
ok ($filename, 'file name is set');
ok ($filename eq '300x300.gif', 'file name is same as original file');
ok (-e 't/uploads/300x300.gif', 'file exists in uploads folder');


# We use an end block to clean up even if the script dies.
END {
  unlink <t/uploads/*>;
  if ($DBH) {
    if ($created_up_table) {
      $DBH->do("DROP SEQUENCE upload_id_seq") if ($drv eq 'Pg');
      $DBH->do("DROP TABLE uploads");
    }
    $DBH->disconnect;
  }
};


